function complete = complete_structure(parameters,list,saveflag,sep)
% COMPLETE_STRUCTURE  Gets the full field list for each sub structure of
%                     parameters
%     complete = complete_structure(parameters,list,saveflag,sep)
%     -----------------------------------------------------------
%     Returns automatically the list cell-structure of parameters and prints
%     the syntax in a text file to be used for building the parameters
%     afterwards.
%
%     INPUT:
%       parameters = .mat parameters file from which to take the structure
%       list       = pre-defined list of {'fieldname','description','class','usertype'}
%                    for each field (can be optained running build_list_v2.m)
%                    PS. list and parameters have to be the same structure
%       saveflag   = flag to save on disk {false}
%       sep        = separator for sub-structures in list struct {'__'}
%                    i.e. list.index---strategy --> sep = '---' if 'strategy' is a
%                         sub-structure of 'index'.
%                         PS. list.match_calib --> 'match_calib' is a field of
%                             'list'. 'calib' is a sub-structure of 'match'.
%
%     OUTPUT:
%       complete = structure "list" template from 'parameters' structure
%

if ~exist('list','var') || isempty(list)
    nolist = true;
else
    nolist = false;
end
if ~exist('saveflag','var') || isempty(saveflag)
    saveflag = false;
end
if ~exist('sep','var') || isempty(sep)
    sep = '__';
end

if saveflag
    disp('Saving txt files with structure list of parameters.mat')
end

names=fieldnames(parameters);
strsep = sep;

for i=1:length(names)
    % level 0
    output=names{i};
    foutput=strrep(output,strsep,'.');
    if nolist
        subnames=fieldnames(parameters.(names{i}));
        list.(output) = cell(size(subnames,1),4);
        for ii=1:length(subnames)
            list.(output){ii,1} = subnames{ii};
        end
    end
    [pippo.(output),tmp]=get_structure(parameters.(names{i}),names{i},saveflag,output,list.(output));
    if ~isempty(tmp)
        tmptmp1=tmp;
        for j=1:length(tmptmp1)
            tmpname1=tmptmp1{j}.name;
            output=[names{i} strsep tmpname1];
            foutput=strrep(output,strsep,'.');
            if nolist
                subnames=fieldnames(parameters.(names{i}).(tmpname1));
                list.(output) = cell(size(subnames,1),4);
                for ii=1:length(subnames)
                    list.(output){ii,1} = subnames{ii};
                end
            end
            % level 1
            [pippo.(output),tmp]=get_structure(parameters.(names{i}).(tmpname1),tmpname1,saveflag,output,list.(output));
            if ~isempty(tmp)
                tmptmp2=tmp;
                for k=1:length(tmptmp2)
                    tmpname2=tmptmp2{k}.name;
                    output=[names{i} strsep tmpname1 strsep tmpname2];
                    foutput=strrep(output,strsep,'.');
                    if nolist
                        subnames=fieldnames(parameters.(names{i}).(tmpname1).(tmpname2));
                        list.(output) = cell(size(subnames,1),4);
                        for ii=1:length(subnames)
                            list.(output){ii,1} = subnames{ii};
                        end
                    end
                    % level 2
                    [pippo.(output),tmp]=get_structure(parameters.(names{i}).(tmpname1).(tmpname2),tmpname2,saveflag,output,list.(output));
                    if ~isempty(tmp)
                        tmptmp3=tmp;
                        for l=1:length(tmptmp3)
                            tmpname3=tmptmp3{l}.name;
                            output=[names{i} strsep tmpname1 strsep tmpname2 strsep tmpname3];
                            foutput=strrep(output,strsep,'.');
                            if nolist
                                subnames=fieldnames(parameters.(names{i}).(tmpname1).(tmpname2).(tmpname3));
                                list.(output) = cell(size(subnames,1),4);
                                for ii=1:length(subnames)
                                    list.(output){ii,1} = subnames{ii};
                                end
                            end
                            % level 3
                            [pippo.(output),tmp]=get_structure(parameters.(names{i}).(tmpname1).(tmpname2).(tmpname3),tmpname3,saveflag,output,list.(output));
                        end
                    end
                end
            end
            
        end
    end
    
end


complete = pippo;

end % end of function
